//
// BunkerMaster - Copyright (c) 2005 Rodrigo Braz Monteiro
// Rodrigo Braz Monteiro's entry for Speedhack 05
// This source file is subject to GNU General Public License
// See included license.txt for more information
//
// You can reach the author at:
// http://zeratul.cellosoft.com
// zeratul@cellosoft.com
//
///////////
// Headers
#include <aldumb.h>
#include <stdlib.h>
#include "audio.h"
#include "core.h"


///////////////
// Constructor
AudioClass::AudioClass() {
}


//////////////
// Destructor
AudioClass::~AudioClass() {
	al_stop_duh(Current);
	for (std::vector<DUH*>::iterator cur=musics.begin();cur!=musics.end();cur++) {
		unload_duh(*cur);
	}
	musics.clear();
	dumb_exit();
}


//////////////
// Initialize
void AudioClass::Init() {
	dumb_register_stdfiles();
	AddMusic("music01.xm");
	AddMusic("music02.xm");
	AddMusic("music03.xm");
	AddMusic("music04.xm");
	AddMusic("music05.xm");

	volume = 0.5f;
	curMusic = -1;
	PlayNext();
}


//////////
// Update
void AudioClass::Update() {
	if (al_poll_duh(Current) != 0) {
		PlayNext();
	}
}


//////////////////
// Play next song
void AudioClass::PlayNext() {
	// Play music
	int newMusic;
	if (musics.size() == 1) newMusic = 0;
	else {
		do {
			newMusic = rand()%musics.size();
		} while (newMusic == curMusic);
	}
	Current = al_start_duh(musics.at(newMusic), 2, 0, volume, 4096, 48000);
	curMusic = newMusic;

	// Dumb weird stuff to make music not loop
	DUH_SIGRENDERER *sr = al_duh_get_sigrenderer(Current);
	DUMB_IT_SIGRENDERER *itsr = duh_get_it_sigrenderer(sr);
	dumb_it_set_loop_callback(itsr, &dumb_it_callback_terminate, NULL);
	dumb_it_set_xm_speed_zero_callback(itsr, &dumb_it_callback_terminate, NULL);

	// Poll
	al_poll_duh(Current);
}


/////////////
// Add music
void AudioClass::AddMusic(std::string name) {
	std::string pathname = "music/" + name;
	char buffer[1024];
	Core.MakeAbsolutePath(pathname.c_str(),buffer);
	musics.push_back(dumb_load_xm_quick(buffer));
}


//////////////
// Get sample
SAMPLE *AudioClass::GetSample(std::string name) {
	std::map<std::string,SAMPLE*>::iterator pos = samples.find(name);
	if (pos != samples.end()) return pos->second;

	char buffer[1024];
	std::string pathname = "/wave/" + name;
	Core.MakeAbsolutePath(pathname.c_str(),buffer);
	SAMPLE *newsample = load_sample(buffer);
	samples[name] = newsample;
	return newsample;
}

//////////////
// Volume up
void AudioClass::VolumeUp()
{
    volume += 0.05f;
    if (volume > 1.0f)
    {
	volume = 1.0f;
    }

    al_duh_set_volume(Current, volume);
 	    
}

//////////////
// Volume down
void AudioClass::VolumeDown()
{
    volume -= 0.05f;
    if (volume < 0.0f)
    {
	volume = 0.0f;
    }

    al_duh_set_volume(Current, volume);
}

//////////
// Global
AudioClass Audio;
