//
// BunkerMaster - Copyright (c) 2005 Rodrigo Braz Monteiro
// Rodrigo Braz Monteiro's entry for Speedhack 05
// This source file is subject to GNU General Public License
// See included license.txt for more information
//
// You can reach the author at:
// http://zeratul.cellosoft.com
// zeratul@cellosoft.com
//
///////////
// Headers
#include "frame_base.h"
#include "sprite.h"
#include "core.h"


///////////////
// Constructor
FrameBase::FrameBase() {
}


//////////////
// Destructor
FrameBase::~FrameBase() {
	// Destroy sprites
	for (std::list<Sprite*>::iterator cur=Sprites.begin();cur!=Sprites.end();cur++) {
		delete *cur;
	}
	Sprites.clear();
}


//////////////
// Add sprite
void FrameBase::AddSprite (Sprite *sprite) {
	Sprites.push_back(sprite);
	sprite->SetParent(this);
}


/////////////////
// Remove sprite
void FrameBase::DestroySprite (Sprite *sprite) {
	Sprites.remove(sprite);
	delete sprite;
}


////////////////
// Draw sprites
void FrameBase::DrawSprites (int pos) {
	using std::list;
	Sprite *current;
	for(list<Sprite*>::iterator cur=Sprites.begin();cur!=Sprites.end();cur++) {
		current = *cur;
		if (current->Position == pos && !current->Hidden) current->Draw();
	}
}


//////////////////
// Update sprites
void FrameBase::UpdateSprites () {
	using std::list;
	list<Sprite*>::iterator next;
	for(list<Sprite*>::iterator cur=Sprites.begin();cur!=Sprites.end();cur = next) {
		next = cur;
		next++;
		(*cur)->Update();
	}
}


///////////////////
// Switch to frame
void FrameBase::Switch(int n) {
	Core.FrameType = n;
	Core.FrameOn = false;
}
