//
// BunkerMaster - Copyright (c) 2005 Rodrigo Braz Monteiro
// Rodrigo Braz Monteiro's entry for Speedhack 05
// This source file is subject to GNU General Public License
// See included license.txt for more information
//
// You can reach the author at:
// http://zeratul.cellosoft.com
// zeratul@cellosoft.com
//
///////////
// Headers
#include <allegro.h>
#include <math.h>
#include "frame_play.h"
#include "input.h"
#include "core.h"
#include "player.h"
#include "soldier_ai.h"
#include "images.h"
#include "building.h"
#include "airdrop.h"
#include "highscores.h"
#include "audio.h"

#ifndef NO_FBLEND
#include <fblend.h>
#endif


///////////////
// Constructor
FramePlay::FramePlay() {
	PlayFrame = this;
	scrollX = -160;
	scrollY = -120;
	AirDrop::NextDrop = 200;
	AirDrop::AirDropping = 0;
	Pause = false;
	GetHigh = false;
	LastCaptured = 0;
	TimeToCapture = 100*60;
	PlayerDeadCount = 0;

	// Create bg
	bg = create_bitmap(320,240);
	double mod;
	for (int i=0;i<240;i++) {
		mod = 1.0 - ((240-i)/2000.0);
		line(bg,0,i,320,i,makecol(int(15*mod),int(52*mod),int(121*mod)));
	}

	// Center terrain
	Terrain.SetCenter(0);

	// Player
	Player = new PlayerSoldier(0);
	AddSoldier(Player);
}


//////////////
// Destructor
FramePlay::~FramePlay() {
	// Destroy soldiers
	for (std::list<Soldier*>::iterator cur=SoldierEnemy.begin();cur!=SoldierEnemy.end();cur++) {
		delete *cur;
	}
	SoldierEnemy.clear();
	for (std::list<Soldier*>::iterator cur=SoldierFriendly.begin();cur!=SoldierFriendly.end();cur++) {
		delete *cur;
	}
	SoldierFriendly.clear();

	// Destroy bg
	destroy_bitmap(bg);
}


//////////
// Update
bool FramePlay::Update() {
	if (key[KEY_ESC]) Switch(0);

	if (Input.KeyDown[Key_Plus])
	{
	    Audio.VolumeUp();
	}
	if (Input.KeyDown[Key_Minus])
	{
	    Audio.VolumeDown();
	}

	// Pause
	if (!Pause && Input.KeyDown[Key_Start]) {
		Pause = true;
		MenuPos = 0;
		return true;
	}

	// Update game
	if (!Pause) {
		// Time to capture
		if (TimeToCapture > 0) TimeToCapture--;
		if (TimeToCapture == 0) {
			Player->Damage(50);
			Player->Life = 0;
			//Player->Alive = false;
		}

		// Refresh random
		srand(Core.Seed + Core.steps);

		// Air drops
		AirDrop::TryAirDrop(this);

		// Update sprites
		UpdateSprites();
		UpdateSoldiers();

		// Scroll on player
		scrollX = int(Player->x-160);
		scrollY = int(Player->y-150);
		if (scrollY > -90) scrollY = -90;
		CurScore = Player->Score;

		// Player dead?
		if (!Player->Alive) {
			PlayerDeadCount++;
			if (PlayerDeadCount > 120) {
				MenuPos = 1;
				Pause = true;
				if (HighScores.ScorePos(CurScore) != -1) GetHighscore();
				return true;
			}
		}
	}

	// Update menu
	else UpdateMenu();

	return true;
}


////////
// Draw
void FramePlay::Draw() {
	// Draw all the stuff
	Terrain.SetCenter(int(Player->x/32));
	DrawBackground();
	DrawSprites(0);
	DrawTerrain();
	DrawSprites(1);
	DrawSoldiers();
	DrawSprites(2);
	DrawInterface();
	DrawMinimap();

	if (Pause) DrawMenu();
}


/////////////
// Interface
void FramePlay::DrawInterface() {
	// Draw interface
	int color = makecol(0,180,0);
	int dx,dy;
	BITMAP *stats = Images.GetImage("stats.bmp");
	dx = 8;
	dy = 8;
	#ifndef NO_FBLEND
	fblend_trans(stats,Core.BackBuffer,dx,dy,200);
	#else
	set_trans_blender(0,0,0,200);
	draw_trans_sprite(Core.BackBuffer,stats,dx,dy);
	#endif
	Core.MarkDirty(dx,dy,160,48);

	// Draw weapons
	int alpha;
	BITMAP *weapon = Images.GetImage("weapon_01.bmp");
	dx = 15;
	dy = 15;
	if (Player->Weapon == 0) alpha = 200;
	else alpha = 100;
	#ifndef NO_FBLEND
	fblend_trans(weapon,Core.BackBuffer,dx,dy,alpha);
	#else
	set_trans_blender(0,0,0,alpha);
	draw_trans_sprite(Core.BackBuffer,weapon,dx,dy);
	#endif
	dx += 19;
	if (Player->HasWeapon[1]) {
		weapon = Images.GetImage("weapon_04.bmp");
		if (Player->Weapon == 1) alpha = 200;
		else alpha = 100;
		#ifndef NO_FBLEND
		fblend_trans(weapon,Core.BackBuffer,dx,dy,alpha);
		#else
		set_trans_blender(0,0,0,alpha);
		draw_trans_sprite(Core.BackBuffer,weapon,dx,dy);
		#endif
	}
	dx -= 19;
	dy += 18;
	if (Player->HasWeapon[2]) {
		weapon = Images.GetImage("weapon_02.bmp");
		if (Player->Weapon == 2) alpha = 200;
		else alpha = 100;
		#ifndef NO_FBLEND
		fblend_trans(weapon,Core.BackBuffer,dx,dy,alpha);
		#else
		set_trans_blender(0,0,0,alpha);
		draw_trans_sprite(Core.BackBuffer,weapon,dx,dy);
		#endif
	}
		dx += 19;
	if (Player->HasWeapon[3]) {
		weapon = Images.GetImage("weapon_03.bmp");
		if (Player->Weapon == 3) alpha = 200;
		else alpha = 100;
		#ifndef NO_FBLEND
		fblend_trans(weapon,Core.BackBuffer,dx,dy,alpha);
		#else
		set_trans_blender(0,0,0,alpha);
		draw_trans_sprite(Core.BackBuffer,weapon,dx,dy);
		#endif
	}

	// Draw score
	textprintf_ex(Core.BackBuffer,font,64,18,color,-1,"Score: ");
	textprintf_right_ex(Core.BackBuffer,font,160,18,color,-1,"%i",CurScore);

	// Draw life
	textprintf_right_ex(Core.BackBuffer,font,160,41,color,-1,"%i",MID(0,int(Player->Life),9999));

	// Draw ammo
	if (Player->Weapon != 0) textprintf_ex(Core.BackBuffer,font,60,41,color,-1,"x %003i",Player->Ammo[Player->Weapon]);
	else textprintf_ex(Core.BackBuffer,font,60,41,color,-1,"x ---");

	// Draw countdown
	textprintf_centre_ex(Core.BackBuffer,font,160,220,makecol(0,255,0),-1,"Time to capture next flagpole: %02i secs",TimeToCapture/60);
	Core.MarkDirty(0,220,320,20);

	// Draw aim
	if (Player->Alive && (!Player->Inside || !Player->Inside->NoShoot)) Player->CrossHair->Draw();
}


///////////
// Minimap
void FramePlay::DrawMinimap() {
	// Collect soldier data for map
	char mapSoldiers[64];
	int start = CellAtPos(Player->x)-32;
	Soldier *current;
	Sprite *currents;
	Building *build;
	int pos;
	for (int i=0;i<64;i++) {
		mapSoldiers[i] = 0;
	}
	using std::list;
	for(list<Sprite*>::iterator cur=Sprites.begin();cur!=Sprites.end();cur++) {
		currents = *cur;
		if (currents->type == 3) {
			build = (Building*) currents;
			pos = int(build->x/32)-start;
			if (pos >= 0 && pos < 64) {
				mapSoldiers[pos] = 2+build->BuildingType;
			}
		}
	}
	for(list<Soldier*>::iterator cur=SoldierEnemy.begin();cur!=SoldierEnemy.end();cur++) {
		current = *cur;
		if (current->Alive) {
			pos = int(current->x/32)-start;
			if (pos >= 0 && pos < 64) mapSoldiers[pos] = 2;
		}
	}
	for(list<Soldier*>::reverse_iterator cur=SoldierFriendly.rbegin();cur!=SoldierFriendly.rend();cur++) {
		current = *cur;
		if (current->Alive) {
			pos = int(current->x/32)-start;
			if (pos >= 0 && pos < 64) mapSoldiers[pos] = 1;
		}
	}

	// Draw minimap
	int dx = 320-128-8-4;
	int dy = 8;
	rect(Core.BackBuffer,dx,dy,dx+131,dy+47,makecol(0,20,0));
	rect(Core.BackBuffer,dx+1,dy+1,dx+130,dy+46,makecol(0,100,0));
	Core.MarkDirty(dx,dy,132,48);
	int color = makecol(0,140,0);
	int col = makecol(0,0,0);
	dx += 2;
	dy = 7;
	for (int i=0;i<64;i++) {
		int height = MID(0,int(GetTerrain(i+start)->height/12+30),44);
		if (i == 32) line(Core.BackBuffer,dx+i*2,dy+46,dx+i*2,dy+height,makecol(0,255,0));
		else line(Core.BackBuffer,dx+i*2,dy+46,dx+i*2,dy+height,color);
		line(Core.BackBuffer,dx+i*2+1,dy+46,dx+i*2+1,dy+height,color);
		if (mapSoldiers[i] != 0) {
			if (mapSoldiers[i] == 1) col = makecol(255,0,0);
			else if (mapSoldiers[i] == 2) col = makecol(0,255,255);
			else if (mapSoldiers[i] == 3) col = makecol(255,255,255);
			else if (mapSoldiers[i] == 4) col = makecol(85,109,10);
			else if (mapSoldiers[i] == 5) col = makecol(213,113,113);
			else if (mapSoldiers[i] == 6) col = makecol(120,0,0);
			rect(Core.BackBuffer,dx+i*2,dy+height-3,dx+i*2+1,dy+height-1,col);
		}
	}
	line(Core.BackBuffer,dx+44,dy+46,dx+44,dy+4,makecol(100,200,100));
	line(Core.BackBuffer,dx+84,dy+46,dx+84,dy+4,makecol(100,200,100));
}


///////////////////
// Updates terrain
TerrainUnit *FramePlay::GetTerrain(int n) {
	if (Terrain.DataAvailable(n)) return Terrain.GetValue(n);
	
	// Not yet set
	TerrainUnit *unit = new TerrainUnit(n);
	Terrain.SetData(n,unit);
	if (Terrain.DataAvailable(n)) return Terrain.GetValue(n);
	else throw "Failed getting terrain unit value";
	//return unit;
}


/////////////////
// Draws terrain
void FramePlay::DrawTerrain() {
	// Get values
	int tileSize = 32;
	int len = screen->w/tileSize;
	int center = int(scrollX / tileSize);
	int n1 = center-1;
	int n2 = center+len+1;
	int dx1,dx2,dy1,dy2;
	BITMAP *grass = Images.GetImage("grass.bmp");
	BITMAP *ground = Images.GetImage("soil.bmp");

	V3D_f v1,v2,v3,v4;
	V3D_f *vertex[4];
	vertex[0] = &v1;
	vertex[1] = &v2;
	vertex[2] = &v3;
	vertex[3] = &v4;
	
	v1.u = 0;
	v1.v = 0;
	v1.z = 0;
	v2.u = 64;
	v2.v = 0;
	v2.z = 0;
	v3.u = 64;
	v3.v = 32;
	v3.z = 0;
	v4.u = 0;
	v4.v = 32;
	v4.z = 0;

	// Draw terrain
	Building *building;
	for (int i=n1;i<=n2;i++) {
		// Draw building
		building = GetTerrain(i)->building;
		if (building) {
			AddSprite(building);
			//building->playFrame = this;
			building->x = i*tileSize;
			building->y = HeightAtPos(i*tileSize);
			GetTerrain(i)->building = NULL;
		}

		// Get coordinates
		dx1 = i*tileSize-int(scrollX);
		dx2 = (i+1)*tileSize-int(scrollX);
		dy1 = int(GetTerrain(i)->height-scrollY)-1;
		dy2 = int(GetTerrain(i+1)->height-scrollY)-1;

		// Draw ground
		v1.v = 0;
		v2.v = 0;
		v3.v = 256;
		v4.v = 256;
		v1.x = dx1;
		v1.y = dy1+10;
		v2.x = dx2;
		v2.y = dy2+10;
		v3.x = dx2;
		v3.y = 300-int(scrollY);
		v4.x = dx1;
		v4.y = 300-int(scrollY);
		polygon3d_f(Core.BackBuffer,POLYTYPE_ATEX_MASK,ground,4,vertex);

		// Draw grass
		v3.v = 32;
		v4.v = 32;
		v1.x = dx1;
		v1.y = dy1;
		v2.x = dx2;
		v2.y = dy2;
		v3.x = dx2;
		v3.y = dy2+20;
		v4.x = dx1;
		v4.y = dy1+20;
		polygon3d_f(Core.BackBuffer,POLYTYPE_ATEX_MASK,grass,4,vertex);

		// Mark dirty
		Core.MarkDirty(dx1,MIN(dy1,dy2),dx2-dx1,600-MIN(dy1,dy2));
	}
}


/////////////////////////////
// Terrain unit constructors
TerrainUnit::TerrainUnit() {
	building = NULL;
}

TerrainUnit::TerrainUnit(int n) {
	building = NULL;
	Generate(n);
}


//////////////
// Destructor
TerrainUnit::~TerrainUnit() {
	//if (building) delete building;
	//building = NULL;
}


/////////////////////
// Get terrain noise
double TerrainUnit::GetNoise(double x,double amplitude,double length,int seed) {
	srand(ABS(seed+int(amplitude*100))%31299);
	double phaseOffset = fmod(double(rand()),6.283185307179586476925); 
	return sin((x*6.283185307179586476925f/length)+phaseOffset)*amplitude;
}


/////////////////////////
// Generate terrain unit
void TerrainUnit::Generate(int n) {
	// Prepare to calculate
	double x = double(n) * 32;
	double fheight = 0;
	double pot = 1.0f;

	// Eight steps of sine wave
	for (int i=0;i<8;i++) {
		srand((i*21+612+Core.Seed)%31362);
		double mod1 = 0.75 + double(rand()&1023)/2048;
		double mod2 = 0.75 + double(rand()&1023)/2048;
		fheight += GetNoise(x,pot*30.0*mod1,pot*1000.0*mod2,Core.Seed);
		pot /= 1.5f;
	}

	// Set height
	height = int(fheight);

	// Generate building
	srand((n*84+15+Core.Seed)%4136);
	building = NULL;
	if (n % 50 == 0) building = new BuildingFlagpole(n > 0);
	else if (n % 2 == 0 && rand() % 15 == 1) building = new BuildingBunker();
	else if (n % 2 == 0 && rand() % 15 == 1) building = new BuildingWatchTower();
	else if (rand() % 8 == 1) building = new BuildingSandbag();

	// Set building
	if (building) {
		building->SetParent(FramePlay::PlayFrame);
	}
}


///////////////////
// Get cell at pos
int FramePlay::CellAtPos(double x) {
	return int(x/32);
}


//////////////
// Get height
double FramePlay::HeightAtPos(double x) {
	int n = CellAtPos(x);
// 	double a = (x - double(n)*32)/32;
// 	return (1-a) * GetTerrain(n)->height + a * GetTerrain(n+1)->height;
	double h1 = GetTerrain(n)->height;
	double d = h1 -GetTerrain(n+1)->height;
	return n*d - (d*x)/32 + h1;
}


///////////
// Draw bg
void FramePlay::DrawBackground() {
	blit(bg,Core.BackBuffer,0,0,0,0,320,240);
}


///////////////
// Add soldier
void FramePlay::AddSoldier(Soldier* soldier) {
	if (soldier->IsEnemy) SoldierEnemy.push_back(soldier);
	else SoldierFriendly.push_back(soldier);
	soldier->SetParent(this);
}


/////////////////
// Draw soldiers
void FramePlay::DrawSoldiers () {
	using std::list;
	for(list<Soldier*>::iterator cur=SoldierEnemy.begin();cur!=SoldierEnemy.end();cur++) {
		if (!(*cur)->Hidden) (*cur)->Draw();
	}
	for(list<Soldier*>::reverse_iterator cur=SoldierFriendly.rbegin();cur!=SoldierFriendly.rend();cur++) {
		if (!(*cur)->Hidden) (*cur)->Draw();
	}
}


///////////////////
// Update soldiers
void FramePlay::UpdateSoldiers () {
	using std::list;
	list<Soldier*>::iterator next;
	for(list<Soldier*>::iterator cur=SoldierFriendly.begin();cur!=SoldierFriendly.end();cur = next) {
		next = cur;
		next++;
		(*cur)->Update();
	}
	for(list<Soldier*>::iterator cur=SoldierEnemy.begin();cur!=SoldierEnemy.end();cur = next) {
		next = cur;
		next++;
		(*cur)->Update();
	}
}


///////////////
// Update menu
void FramePlay::UpdateMenu() {
	// Highscores
	if (GetHigh) {
		if (Input.KeyDown[Key_Start]) {
			GetHigh = false;
			HighScores.Insert(CurScore,PlayerName);
		}
		if (Input.Key[Key_Up]) CursorHold++;
		else if (Input.Key[Key_Down]) CursorHold++;
		else CursorHold = 0;
		if (Input.KeyDown[Key_Up] || (CursorHold > 30 && CursorHold % 5 == 0 && Input.Key[Key_Up])) {
			PlayerName[NameCur]++;
			if (PlayerName[NameCur] > 'Z') PlayerName[NameCur] = 'Z';
		}
		if (Input.KeyDown[Key_Down] || (CursorHold > 30 && CursorHold % 5 == 0 && Input.Key[Key_Down])) {
			PlayerName[NameCur]--;
			if (PlayerName[NameCur] < 'A') PlayerName[NameCur] = 'A';
		}
		if (Input.KeyDown[Key_A]) {
			NameCur++;
			if (NameCur == 15) NameCur = 14;
			else PlayerName[NameCur] = PlayerName[NameCur-1];
		}
		if (Input.KeyDown[Key_B]) {
			NameCur--;
			if (NameCur < 0) NameCur = 0;
			else PlayerName[NameCur+1] = 0;
		}
	}

	// Normal menu
	else {
		// Accept
		if (Input.KeyDown[Key_A]) {
			if (MenuPos == 0) Pause = false;
			if (MenuPos == 1) Switch(1);
			if (MenuPos == 2) Switch(2);
			if (MenuPos == 3) Switch(0);
		}

		// Cancel
		int start = 0;
		if (!Player->Alive) start = 1;
		if (start == 0 && (Input.KeyDown[Key_Start] || Input.KeyDown[Key_B])) Pause = false;

		// Choose option
		int menuItems = 4;
		if (Input.KeyDown[Key_Down]) MenuPos++;
		if (Input.KeyDown[Key_Up]) MenuPos--;
		if (MenuPos < start) MenuPos = menuItems-1;
		if (MenuPos >= menuItems) MenuPos = start;
	}

	if (Input.KeyDown[Key_Plus])
	{
	    Audio.VolumeUp();
	}
	if (Input.KeyDown[Key_Minus])
	{
	    Audio.VolumeDown();
	}

}


/////////////
// Draw menu
void FramePlay::DrawMenu() {
	// Draw rectangle
	int width;
	if (GetHigh) width = 320;
	else width = 240;
	int height = 80;
	int dx = (320-width)/2;
	int dy = (240-height)/2;
	#ifndef NO_FBLEND
	fblend_rect_trans (Core.BackBuffer,dx+2,dy+2,width-3,height-3,makecol(0,100,0),192);  
	#else
	set_trans_blender(0,0,0,192);
	drawing_mode(DRAW_MODE_TRANS,NULL,0,0);
	rectfill(Core.BackBuffer,dx+2,dy+2,dx+width-2,dy+height-2,makecol(0,100,0));
	drawing_mode(DRAW_MODE_SOLID,NULL,0,0);
	#endif
	rect(Core.BackBuffer,dx,dy,dx+width,dy+height,makecol(0,20,0));
	rect(Core.BackBuffer,dx+1,dy+1,dx+width-1,dy+height-1,makecol(0,100,0));
	Core.MarkDirty(dx,dy,width,height);

	// Highscore
	if (GetHigh) {
		// Prompt
		dx = 160;
		dy = 93;
		int step = 12;
		int color = makecol(0,200,0);
		textout_centre_ex(Core.BackBuffer, font, "Highscore! Enter your name:", dx, dy, color, -1);
		dy += step;
		textout_centre_ex(Core.BackBuffer, font, "Direction to choose, Start to accept,", dx, dy, color, -1);
		dy += step;
		textout_centre_ex(Core.BackBuffer, font, "Button 'A' to accept, 'B' to return.", dx, dy, color, -1);
		dy += 24;

		// Name
		char buffer1[16];
		char buffer2[16];
		strcpy(buffer1,PlayerName);
		strcpy(buffer2,PlayerName);
		for (int i=0;i<NameCur;i++) buffer2[i] = ' ';
		if (NameCur < 15) buffer1[NameCur] = ' ';
		textprintf_centre_ex(Core.BackBuffer,font,dx,dy,makecol(0,255,0),-1,"%s",buffer1);
		textprintf_centre_ex(Core.BackBuffer,font,dx,dy,makecol(255,255,255),-1,"%s",buffer2);
	}

	// Normal menu
	else {
		// Options
		dx = 160;
		dy = 93;
		int step = 12;
		int color = makecol(0,200,0);
		textout_centre_ex(Core.BackBuffer, font, "Press Button 'A' to choose:", dx, dy, color, -1);
		dy += 20;
		if (!Player->Alive) color = makecol(128,128,128);
		else if (MenuPos == 0) color = makecol(255,255,255);
		else color = makecol(0,255,0);
		textout_centre_ex(Core.BackBuffer, font, "Resume", dx, dy, color, -1);
		dy += step;
		if (MenuPos == 1) color = makecol(255,255,255);
		else color = makecol(0,255,0);
		textout_centre_ex(Core.BackBuffer, font, "Start another level", dx, dy, color, -1);
		dy += step;
		if (MenuPos == 2) color = makecol(255,255,255);
		else color = makecol(0,255,0);
		textout_centre_ex(Core.BackBuffer, font, "Exit to Main Menu", dx, dy, color, -1);
		dy += step;
		if (MenuPos == 3) color = makecol(255,255,255);
		else color = makecol(0,255,0);
		textout_centre_ex(Core.BackBuffer, font, "Exit Game", dx, dy, color, -1);
		dy += step;
	}
}


/////////////////
// Get highscore
void FramePlay::GetHighscore() {
	GetHigh = true;
	NameCur = 0;
	CursorHold = 0;
	for (int i=0;i<16;i++) PlayerName[i] = 0;
	PlayerName[0] = 'A';
}


//////////
// Static
FramePlay *FramePlay::PlayFrame;
