//
// BunkerMaster - Copyright (c) 2005 Rodrigo Braz Monteiro
// Rodrigo Braz Monteiro's entry for Speedhack 05
// This source file is subject to GNU General Public License
// See included license.txt for more information
//
// You can reach the author at:
// http://zeratul.cellosoft.com
// zeratul@cellosoft.com
//
///////////
// Headers
#include <allegro.h>
#include "frame_title.h"
#include "input.h"
#include "core.h"
#include "highscores.h"
#include "images.h"
#include "audio.h"


///////////////
// Constructor
FrameTitle::FrameTitle() {
	timer = 0;
}


//////////////
// Destructor
FrameTitle::~FrameTitle() {
}


//////////
// Update
bool FrameTitle::Update() {
	timer++;
	if (Input.KeyDown[Key_Start] || Input.KeyDown[Key_A]) {
		Switch(1);
		return true;
	}
	if (Input.KeyDown[Key_B] || key[KEY_ESC]) {
		Switch(0);
		return true;
	}
	if (Input.KeyDown[Key_Plus])
	{
	    Audio.VolumeUp();
	    return true;
	}
	if (Input.KeyDown[Key_Minus])
	{
	    Audio.VolumeDown();
	    return true;
	}
	return true;
}


////////
// Draw
void FrameTitle::Draw() {
	// Setup
	clear_bitmap(Core.BackBuffer);
	Core.MarkDirty(0,0,320,240);

	// Logo
	BITMAP *logo = Images.GetImage("logo.bmp");
	draw_sprite(Core.BackBuffer,logo,10,20);
	logo = Images.GetImage("cellosoft.bmp");
	draw_sprite(Core.BackBuffer,logo,200-128,200-128);

	// Start flashing thing
	int color = makecol(0,200,0);
	if ((timer/16)%2 == 0) color = makecol(255,255,255);
	textout_centre_ex(Core.BackBuffer, font, "Press Button 'A' to start or 'B' to quit", 160, 80, color, -1);

	// Instructions
	color = makecol(0,200,0);
	int dy = 100;
	if ((time(NULL)/10)%2)
	{
	    textout_centre_ex(Core.BackBuffer, font, "Welcome to Bunkermaster v1.01", 160, dy, color, -1);
	    int dx = 2;
	    dy += 20;
	    textout_ex(Core.BackBuffer, font, "     A - Ctrl  - But4  - Shoot", dx, dy, color, -1);
	    dy += 12;
	    textout_ex(Core.BackBuffer, font, "     B - Space - But1  - Enter/Leave/", dx, dy, color, -1);
	    dy += 12;
	    textout_ex(Core.BackBuffer, font, "                      Capture buildings", dx, dy, color, -1);
	    dy += 12;
	    textout_ex(Core.BackBuffer, font, "Select - Tab   - But7  - Change weapons", dx, dy, color, -1);
	    dy += 12;
	    textout_ex(Core.BackBuffer, font, " Start - Enter - But8  - Menu", dx, dy, color, -1);
	    dy += 12;
	    textout_ex(Core.BackBuffer, font, "  Left - Left  - Left  - Move left", dx, dy, color, -1);
	    dy += 12;
	    textout_ex(Core.BackBuffer, font, " Right - Right - Right - Move right", dx, dy, color, -1);
	    dy += 12;
	    textout_ex(Core.BackBuffer, font, "    Up - Up    - Up    - Aim up", dx, dy, color, -1);
	    dy += 12;
	    textout_ex(Core.BackBuffer, font, "  Down - Down  - Down  - Aim down", dx, dy, color, -1);
	    dy += 12;
	}
	else
	{
	    // Highscores
	    int dx = 70;
	    textout_ex(Core.BackBuffer, font, "Highscores:", dx, dy, color, -1);
	    dx += 10;
	    dy += 15;
	    for (int i=0;i<5;i++) {
		textprintf_ex(Core.BackBuffer,font,dx,dy,color,-1,"%02i. %s",i+1,HighScores.name[i]);
		textprintf_ex(Core.BackBuffer,font,dx+160,dy,color,-1,"- %6i",HighScores.score[i]);
		textprintf_ex(Core.BackBuffer,font,dx+250,dy,color,-1,"%02i. %s",i+6,HighScores.name[i+5]);
		textprintf_ex(Core.BackBuffer,font,dx+250+160,dy,color,-1,"- %6i",HighScores.score[i+5]);
		dy += 12;
	    }

	    // Copyright
	    textout_centre_ex(Core.BackBuffer, font, "Bunkermaster - Copyright (c) 2005", 160, 180, color, -1);
	    textout_centre_ex(Core.BackBuffer, font, "Rodrigo Braz Monteiro.", 160, 190, color, -1);
	    textout_centre_ex(Core.BackBuffer, font, "This game is distributed under the terms", 160, 200, color, -1);
	    textout_centre_ex(Core.BackBuffer, font, "of the GNU GPL. See license.txt.", 160, 210, color, -1);
	    textout_centre_ex(Core.BackBuffer, font, "Cellosoft and its logo are Copyright", 160, 220, color, -1);
	    textout_centre_ex(Core.BackBuffer, font, "(c) 1995-2005 - Marcello Bastea-Forte.", 160, 230, color, -1);
	}
}
