//
// BunkerMaster - Copyright (c) 2005 Rodrigo Braz Monteiro
// Rodrigo Braz Monteiro's entry for Speedhack 05
// This source file is subject to GNU General Public License
// See included license.txt for more information
//
// You can reach the author at:
// http://zeratul.cellosoft.com
// zeratul@cellosoft.com
//
///////////
// Headers
#include <math.h>
#include <allegro.h>
#include "frame_play.h"
#include "particles.h"
#include "images.h"
#include "soldier.h"


///////////////
// Constructor
Particle::Particle() {
	TimeToLive = -1;
	Damages = false;
	Friendly = false;
	DieOnHit = false;
	Damage = 0;
	type = 2;
	Fade = false;
	Explosion = 0;
}


////////////////////////////////////
// Copy constructor from blueprints
Particle::Particle(int n) {
	Particle *from = &Blueprints[n];
	TimeToLive = from->TimeToLive;
	Damages = from->Damages;
	Damage = from->Damage;
	Friendly = from->Friendly;
	DieOnHit = from->DieOnHit;
	bmp = from->bmp;
	Friendly = from->Friendly;
	pivotX = from->pivotX;
	pivotY = from->pivotY;
	GroundDrag = from->GroundDrag;
	GravityEffect = from->GravityEffect;
	BackOnHit = from->BackOnHit;
	HeightSupport = from->HeightSupport;
	drawMode = from->drawMode;
	Fade = from->Fade;
	Bounce = from->Bounce;
	Explosion = from->Explosion;
	DragX = from->DragX;
	DragY = from->DragY;
}


//////////
// Update
void Particle::Update() {
	// Time to live
	if (TimeToLive != -1) {
		TimeToLive--;
		if (TimeToLive == 0) {
			//Explode();
			Destroy();
			return;
		}
	}

	// Check for collision
	if (Damages) {
		Soldier *current;

		// Friendly fire
		if (Friendly) {
			for (std::list<Soldier*>::iterator cur=playFrame->SoldierEnemy.begin();cur!=playFrame->SoldierEnemy.end();cur++) {
				current = (*cur);
				if (current->Alive && current->CollidesWith(this)) {
					current->Damage(Damage,parent);
					if (DieOnHit) {
						Destroy();
						return;
					}
				}
			}
		}

		// Enemy fire
		else {
			for (std::list<Soldier*>::iterator cur=playFrame->SoldierFriendly.begin();cur!=playFrame->SoldierFriendly.end();cur++) {
				current = (*cur);
				if (current->Alive && current->CollidesWith(this)) {
					current->Damage(Damage,parent);
					if (DieOnHit) {
						Destroy();
						return;
					}
				}
			}
		}
	}

	// Fade
	if (Fade) {
		alpha = TimeToLive*255/Fade;
	}

	Object::Update();
}


///////////
// Explode
void Particle::Explode() {
	if (Explosion) {
		for (int i=0;i<Explosion*4;i++) {
			// Spawn smoke
			Particle *part = SpawnParticle(Particle_Smoke_02,0,0);
			part->x += (rand()%(Explosion*2+1))-Explosion;
			part->y += (rand()%(Explosion*2+1))-Explosion;
			part->vx = ((rand()%(Explosion*2+1))-Explosion)*0.3;
			part->vy = ((rand()%(Explosion*2+1))-Explosion)*0.3;

			// Spawn fire
			part = SpawnParticle(Particle_Explosion_01,0,0);
			part->Friendly = Friendly;
			part->x += (rand()%(Explosion*2+1))-Explosion;
			part->y += (rand()%(Explosion*2+1))-Explosion;
			part->vx = ((rand()%(Explosion*2+1))-Explosion)*0.3;
			part->vy = ((rand()%(Explosion*2+1))-Explosion)*0.3;
		}

		// Play sound
		PlaySound("explosion.wav");
		PlaySound("explosion.wav");

		// Damage area
		using std::list;
		list<Soldier*>::iterator begin;
		list<Soldier*>::iterator end;
		if (Friendly) {
			begin = playFrame->SoldierEnemy.begin();
			end = playFrame->SoldierEnemy.end();
		}
		else {
			begin = playFrame->SoldierFriendly.begin();
			end = playFrame->SoldierFriendly.end();
		}
		double dist;
		double radius = pow(Explosion*7,2);
		double effect;
		Soldier *cur;
		for (;begin!=end;begin++) {
			cur = *begin;
			dist = pow((cur->x - x),2) + pow((cur->y - y),2);
			if (dist < radius) {
				effect = MID(0,1-pow((dist/radius),3),1);
				(*begin)->Damage(effect*Explosion*5,parent);
				if (!cur->Alive) cur->Bounce = 0.5;
				double vel = effect*Explosion*0.6;
				double angle = atan2(cur->y-y,cur->x-x);
				if (!cur->Inside) {
					if (cur->Life > 0) cur->SetAnimSequence(0);
					cur->y -= 5;
					cur->vx += vel*cos(angle);
					cur->vy -= vel*sin(angle);
					cur->x += vx;
					cur->y += vy;
				}
			}
		}
	}
}


///////////
// Destroy
void Particle::Destroy() {
	Explode();
	Object::Destroy();
}


//////////////
// Blueprints
std::vector<Particle> Particle::Blueprints;


///////////////////
// Init blueprints
void Particle::InitBlueprints() {
	Particle cur;

	// Friendly shot
	cur.HeightSupport = 3;
	cur.BackOnHit = true;
	cur.bmp = Images.GetImage("friendly_bullet.bmp");
	cur.Damages = true;
	cur.Damage = 8;
	cur.DieOnHit = true;
	cur.TimeToLive = 22;
	cur.pivotX = 15;
	cur.pivotY = 1;
	cur.Friendly = true;
	cur.GravityEffect = 0.3;
	Blueprints.push_back(cur);

	// Enemy shot
	cur.bmp = Images.GetImage("enemy_bullet.bmp");
	cur.Friendly = false;
	Blueprints.push_back(cur);

	// Shells
	cur.pivotX = 3;
	cur.pivotY = -2; 
	cur.Friendly = true;
	cur.Damages = false;
	cur.TimeToLive = 350;
	cur.DieOnHit = false;
	cur.GravityEffect = 1.0;
	cur.GroundDrag = 1.0;
	cur.bmp = Images.GetImage("shell.bmp");
	Blueprints.push_back(cur);

	// Blood
	cur.pivotX = 2;
	cur.pivotY = -2;
	cur.TimeToLive = 200;
	cur.bmp = Images.GetImage("blood01.bmp");
	Blueprints.push_back(cur);
	cur.bmp = Images.GetImage("blood02.bmp");
	Blueprints.push_back(cur);
	cur.bmp = Images.GetImage("blood03.bmp");
	Blueprints.push_back(cur);

	// Fire
	cur.Bounce = 0.8;
	cur.drawMode = 1;
	cur.pivotX = 5;
	cur.pivotY = 5;
	cur.TimeToLive = 60;
	cur.GravityEffect = -0.1;
	cur.bmp = Images.GetImage("fireball.bmp");
	cur.Damage = 0.05;
	cur.Damages = true;
	cur.GroundDrag = 0;
	cur.Fade = 60;
	Blueprints.push_back(cur);

	// Explosion
	cur.pivotX = 16;
	cur.pivotY = 16;
	cur.TimeToLive = 20;
	cur.bmp = Images.GetImage("explosion_01.bmp");
	cur.Damage = 0.0;
	cur.Damages = false;
	cur.Fade = 20;
	Blueprints.push_back(cur);

	// Smoke
	cur.pivotX = 5;
	cur.pivotY = 5;
	cur.drawMode = 2;
	cur.TimeToLive = 100;
	cur.GravityEffect = -0.25;
	cur.bmp = Images.GetImage("smoke01.bmp");
	cur.Damage = 0;
	cur.Damages = false;
	cur.Fade = 120;
	Blueprints.push_back(cur);

	// More smoke
	cur.bmp = Images.GetImage("smoke02.bmp");
	cur.pivotX = 10;
	cur.pivotY = 10;
	cur.TimeToLive = 30;
	Blueprints.push_back(cur);

	// Grenade
	cur.HeightSupport = 0;
	cur.drawMode = 0;
	cur.TimeToLive = 80;
	cur.Bounce = 0.7;
	cur.GravityEffect = 1;
	cur.bmp = Images.GetImage("grenade.bmp");
	cur.Damage = 0;
	cur.Damages = false;
	cur.Fade = 0;
	cur.Explosion = 10;
	Blueprints.push_back(cur);

	// Rocket
	cur.HeightSupport = 9;
	cur.GravityEffect = 1.5;
	cur.pivotX = 3;
	cur.pivotY = 1;
	cur.TimeToLive = 120;
	cur.Damage = 30;
	cur.Damages = true;
	cur.Explosion = 20;
	cur.DieOnHit = true;
	cur.bmp = Images.GetImage("rocket.bmp");
	Blueprints.push_back(cur);

	// Enemy bomb
	cur.DragX = 0.99;
	cur.DragY = 0.95;
	cur.GravityEffect = 0.7;
	cur.pivotX = 8;
	cur.pivotY = 26;
	cur.TimeToLive = 0;
	cur.Damages = false;
	cur.Explosion = 8;
	cur.Friendly = false;
	cur.bmp = Images.GetImage("enemy_bomb.bmp");
	Blueprints.push_back(cur);

	// Friendly bomb
	cur.Friendly = true;
	cur.bmp = Images.GetImage("friendly_bomb.bmp");
	Blueprints.push_back(cur);
}
