//
// BunkerMaster - Copyright (c) 2005 Rodrigo Braz Monteiro
// Rodrigo Braz Monteiro's entry for Speedhack 05
// This source file is subject to GNU General Public License
// See included license.txt for more information
//
// You can reach the author at:
// http://zeratul.cellosoft.com
// zeratul@cellosoft.com
//


#ifndef ROTATING_CACHE_H
#define ROTATING_CACHE_H


///////////////////////////
// Rotating Cache template
template <class CacheType,int CacheSize>
class RotatingCache {
private:
	CacheType value [CacheSize];
	bool isSet [CacheSize];
	bool init;
	int offset;
	int start;

	int GetIndex(int n);

public:
	RotatingCache();

	CacheType GetValue(int n);
	bool DataAvailable(int n);
	void SetStart(int n);
	void SetCenter(int n);
	void SetData(int n,CacheType x);
};


///////////////
// Constructor
template <class CacheType,int CacheSize>
RotatingCache<CacheType,CacheSize>::RotatingCache() {
	for (int i=0;i<CacheSize;i++) {
		isSet[i] = false;
	}
	offset = 0;
	start = 0;
	init = false;
}


////////////////////////////////////////////////////////
// Gets what position on the array a certain element is
template <class CacheType,int CacheSize>
int RotatingCache<CacheType,CacheSize>::GetIndex(int n) {
	if (n < start || n >= start + CacheSize) return -1;
	int temp = (n - start + offset);
	int final = temp%CacheSize;
	return final;
}


///////////////////////
// Sets start position
template <class CacheType,int CacheSize>
void RotatingCache<CacheType,CacheSize>::SetStart(int n) {
	// Prepare, check, and set offset
	int moveBy = n-start;
	if (moveBy == 0) return;
	int pos;

	// Moved too much, erase all of it
	if (moveBy >= CacheSize || moveBy <= -CacheSize) {
		for (int i=0;i<CacheSize;i++) isSet[i] = false;
	}

	// Moved right, erase first elements
	else if (moveBy > 0) {
		for (int i=start;i<start+moveBy;i++) {
			pos = GetIndex(i);
			if (pos != -1) isSet[pos] = false;
		}
	}

	// Moved left, erase last elements
	else {
		int endLoop = start+CacheSize;
		int startLoop = start+CacheSize+moveBy;
		for (int i=startLoop;i<endLoop;i++) {
			pos = GetIndex(i);
			if (pos != -1) isSet[pos] = false;
		}
	}

	// Finally, set new start position
	start = n;
	while (offset < 0) offset += CacheSize;
	offset = (offset+moveBy+CacheSize)%CacheSize;
	init = true;
}


////////////////////////
// Sets center position
template <class CacheType,int CacheSize>
void RotatingCache<CacheType,CacheSize>::SetCenter(int n) {
	SetStart(n-(CacheSize/2));
}


/////////////////////////
// Sets data at position
template <class CacheType,int CacheSize>
void RotatingCache<CacheType,CacheSize>::SetData(int n,CacheType x) {
	// Centers cache on value if it's not initialized
	if (!init) SetCenter(n);

	// Stores if it can
	int pos = GetIndex(n);
	if (pos != -1) {
		value[pos] = x;
		isSet[pos] = true;
	}
}


/////////////////////////
// Gets data at position
template <class CacheType,int CacheSize>
CacheType RotatingCache<CacheType,CacheSize>::GetValue(int n) {
//	if (DataAvailable(n)) return value[GetIndex(n)];
//	throw "Attempted to read undefined cache value";
	return value[GetIndex(n)];
}


////////////////////////////////////////////////
// Checks if it's OK to get data from position
template <class CacheType,int CacheSize>
bool RotatingCache<CacheType,CacheSize>::DataAvailable(int n) {
	if (n < start || n >= start + CacheSize) return false;
	return isSet[GetIndex(n)];
}


#endif
