//
// BunkerMaster - Copyright (c) 2005 Rodrigo Braz Monteiro
// Rodrigo Braz Monteiro's entry for Speedhack 05
// This source file is subject to GNU General Public License
// See included license.txt for more information
//
// You can reach the author at:
// http://zeratul.cellosoft.com
// zeratul@cellosoft.com
//
///////////
// Headers
#include <math.h>
#include <allegro.h>
#include "soldier.h"
#include "frame_play.h"
#include "images.h"
#include "particles.h"
#include "player.h"
#include "animation.h"
#include "building.h"


///////////////
// Constructor
Soldier::Soldier(double _x,bool enemy) {
	// Set vars
	HeightSupport = 0;
	IsEnemy = enemy;
	x = _x;
	y = 1000;	// Will be fixed by Update()
	speed = 2;
	pivotX = 8;
	pivotY = 15;
	Direction = (rand()%2)*2-1;
	shootCooldown = 0;
	Alive = true;
	angle = 0;
	type = 1;
	Weapon = 0;
	Hidden = false;
	Inside = NULL;
	IsAI = false;
	DamageMultiply = 1;
	Bounce = 0;

	// Weapon setup
	HasWeapon[0] = true;
	Ammo[0] = -1;
	for (int i=1;i<4;i++) {
		HasWeapon[i] = false;
		Ammo[i] = 0;
	}

	// Set image
	if (IsEnemy) anim = Animation::GetAnimation("enemy_soldier");
	else anim = Animation::GetAnimation("friendly_soldier");
}


//////////////
// Destructor
Soldier::~Soldier() {
}


//////////
// Update
void Soldier::Update() {
	// Counters
	if (shootCooldown > 0) shootCooldown--;
	if (timer > 0) timer--;

	// Kill if too far from player
	if (ABS(playFrame->Player->x - x) > 5000) Die();

	// Stop if inside building
	if (Inside) {
		vx = vy = 0;
	}

	// Destroy
	if (!Alive && timer == 0) {
		Destroy();
		return;
	}

	Object::Update();
}


////////
// Move
void Soldier::Move(double fdir) {
	// Get data
	int dir = 1;
	if (fdir < 0) dir = -1;
	//double fspeed = speed*fdir*dir;
	double height = playFrame->HeightAtPos(x);

	// Check contact
	if (!Inside && IsOnGround()) {
		double newHeight = playFrame->HeightAtPos(x+fdir*speed);
		double mod = MID(-speed*0.4,(newHeight-height)*0.5,speed*0.2);
		double newSpeed = (speed+mod)*ABS(fdir);
		vx = MID(-newSpeed,vx + (newSpeed*dir/10.0),newSpeed);
		vy = 0;
		y = playFrame->HeightAtPos(x+vx);
		SetAnimSequence(1);
	}

	// Set animation
	Direction = dir;
	FlipHorizontal = (dir == -1);
}


////////
// Stop
void Soldier::Stop() {
	if (vx < 10) vx *= 0.8;
	else vx *= 0.99;
	if (Alive) SetAnimSequence(0);
}


////////
// Jump
void Soldier::Jump() {
	vy -= 5;
	vx *= 1.5;
	SetAnimSequence(0);
}


/////////
// Shoot
void Soldier::Shoot() {
	if (Inside && Inside->NoShoot) return;

	if (shootCooldown == 0) {
		// Machinegun
		if (Weapon == 0) {
			// Set cooldown
			shootCooldown = 6;

			// Spawn bullet
			int type;
			if (IsEnemy) type = Particle_Enemy_Bullet;
			else type = Particle_Friendly_Bullet;
			Particle *part = SpawnParticle(type,14,11);
			double bulletspeed = 20+(rand()%21)/10.0;
			part->vx = bulletspeed*Direction*cos(angle);
			part->vy = -bulletspeed*sin(angle);
			part->FlipHorizontal = FlipHorizontal;

			// Spawn shell
			part = SpawnParticle(Particle_Bullet_Shells,10,11);
			part->vx = -(((rand()%15)/10.0)+1)*Direction + vx*0.5;
			part->vy = -(((rand()%15)/10.0)+1) + vy*0.5;
			part->HeightSupport += (rand()%3)-1;
			part->TimeToLive += rand()%100;

			// Play sound
			PlaySound("machine.wav");

			// Anim
			SetAnimSequence(2);
		}

		// Grenade
		if (Weapon == 1) {
			// Set cooldown
			shootCooldown = 30;
			double bulletspeed = 5+(rand()%100)/100.0;
			Particle *part = SpawnParticle(Particle_Grenade,14,11);
			part->x += (rand()%5)-2;
			part->y += (rand()%5)-2;
			part->vx = bulletspeed*Direction*cos(angle)+vx;
			part->vy = -bulletspeed*sin(angle);
			PlaySound("grenade.wav");
		}

		// Flamethrower
		if (Weapon == 2) {
			// Set cooldown
			shootCooldown = 0;

			for (int i=0;i<3;i++) {
				// Spawn smoke
				double bulletspeed = 2+(rand()%200)/100.0;
				double ang = angle + (rand()%500)/1000.0 - 0.25;
				Particle *part = SpawnParticle(Particle_Smoke_01,21,13);
				part->x += (rand()%5)-2;
				part->y += (rand()%5)-2;
				part->vx = bulletspeed*Direction*cos(ang)+vx;
				part->vy = -bulletspeed*sin(ang);

				// Spawn bullet
				ang = angle + (rand()%500)/1000.0 - 0.25;
				part = SpawnParticle(Particle_Fire_01,21,13);
				part->Friendly = !IsEnemy;
				part->x += (rand()%5)-2;
				part->y += (rand()%5)-2;
				part->vx = bulletspeed*Direction*cos(ang)+vx;
				part->vy = -bulletspeed*sin(ang);
			}

			// Sound
			PlaySound("flame.wav");
			
			// Anim
			SetAnimSequence(2);
		}

		// Rocket launcher
		if (Weapon == 3) {
			// Set cooldown
			shootCooldown = 120;
			double bulletspeed = 12+(rand()%100)/100.0;
			Particle *part = SpawnParticle(Particle_Rocket,14,11);
			part->x += (rand()%5)-2;
			part->y += (rand()%5)-2;
			part->vx = bulletspeed*Direction*cos(angle)+vx;
			part->vy = -bulletspeed*sin(angle);

			// Sound
			PlaySound("rocket.wav");

			// Anim
			SetAnimSequence(2);
		}

		// Check ammo
		if (Weapon > 0) {
			Ammo[Weapon]--;
			if (Ammo[Weapon] == 0) {
				HasWeapon[Weapon] = false;
				NextWeapon();
			}
		}
	}
}


///////
// Die
void Soldier::Die() {
	if (Inside) Inside->LeaveBuilding(this);
	if (Alive) {
		Alive = false;
		timer = 240;
		PlaySound("death.wav");
		SetAnimSequence(3);
	}
}


///////////////
// Take Damage
void Soldier::Damage(double amount,Object *From) {
	// Check if it's alive and cause damage
	if (!Alive) return;
	double totalDamage = amount*DamageMultiply;
	if (totalDamage < 0.01) return;
	Life -= totalDamage;

	// Spawn blood
	int n = 0;
	if (totalDamage >= 1) n = MID(1,int(totalDamage),8);
	else if (rand() % 100 < int(totalDamage*100)) n = 1;
	for (int i=0;i<n;i++) {
		Particle *part = SpawnParticle(Particle_Blood_01+(rand()%3),8,8);
		part->vx = -(((rand()%81)/20.0)-2.0) + vx*0.5;
		part->vy = -(((rand()%25)/10.0)+1) + vy*0.5;
		part->pivotY += rand()%2;
		part->TimeToLive += rand()%100;
		part->HeightSupport += (rand()%3)-1;
	}

	// Killed?
	if (Life <= 0) {
		Die();
		if (From == playFrame->Player) {
			playFrame->Player->Score += PointsWorth;
		}
	}

	// Score
	if (From == playFrame->Player) {
		playFrame->Player->Score += int(totalDamage * 3);
	}
}


////////////////
// Heal soldier
void Soldier::Heal(double amount) {
	Life += amount;
	if (Life > MaxLife) Life = MaxLife;
}


//////////////////
// Destroy object
void Soldier::Destroy() {
	if (Inside) Inside->LeaveBuilding(this);
	if (this != playFrame->Player) {
		if (IsEnemy) playFrame->SoldierEnemy.remove(this);
		else playFrame->SoldierFriendly.remove(this);
		delete this;
	}
}


//////////////////////////////////
// Get building it's over, if any
Building *Soldier::OverBuilding() {
	Building* current;
	for(std::list<Sprite*>::iterator cur=parent->Sprites.begin();cur!=parent->Sprites.end();cur++) {
		if ((*cur)->type == 3) {
			current = (Building*) (*cur);
			if (CollidesWith(current)) {
				return current;
			}
		}
	}
	return NULL;
}


////////////////////
// Capture building
void Soldier::Capture(Building *building) {
	building->CaptureAttempt(this);
}


///////////////////
// Get next weapon
void Soldier::NextWeapon() {
	for (int i=Weapon+1;i<4;i++) {
		if (HasWeapon[i]) {
			Weapon=i;
			if (this == playFrame->Player) PlaySound("switch.wav");
			return;
		}
	}
	if (Weapon != 0) {
		if (this == playFrame->Player) PlaySound("switch.wav");
		Weapon = 0;
	}
}


///////////////////
// Get next weapon
bool Soldier::HasBetterWeapon() {
	for (int i=Weapon+1;i<4;i++) {
		if (HasWeapon[i]) {
			return true;
		}
	}
	return false;
}
