//
// BunkerMaster - Copyright (c) 2005 Rodrigo Braz Monteiro
// Rodrigo Braz Monteiro's entry for Speedhack 05
// This source file is subject to GNU General Public License
// See included license.txt for more information
//
// You can reach the author at:
// http://zeratul.cellosoft.com
// zeratul@cellosoft.com
//
///////////
// Headers
#include <math.h>
#include <allegro.h>
#include <stdlib.h>
#include "soldier_ai.h"
#include "frame_play.h"
#include "building.h"
#include "collectible.h"
#include "airdrop.h"


///////////////
// Constructor
SoldierAI::SoldierAI(double x,bool isEnemy,int level) : Soldier(x,isEnemy) {
	curMode = AI_None;
	IsAI = true;
	FavoriteDistance = rand()%40+60;
	MaxLife = 10+level/2.0;
	Life = 10+level/2.0;
	PointsWorth = 50+level*25;
	LeaveIn = -1;
	if (rand()%30 < -3+level) HasWeapon[1] = true;
	if (rand()%40 < -7+level) HasWeapon[2] = true;
	if (rand()%50 < -10+level) HasWeapon[3] = true;
}


//////////
// Update
void SoldierAI::Update() {
	// Dead?
	if (!Alive) {
		Stop();
		Soldier::Update();
		return;
	}

	// No AI Set
	if (curMode == AI_None) {
		SetAIMode(AI_Watch);
	}

	// Leave building?
	if (LeaveIn != -1) {
		LeaveIn--;
		if (Inside) {
			if (LeaveIn < 0) {
				Inside->LeaveBuilding(this);
				LeaveIn = 50;
			}
		}
	}

	// Weapon
	if (HasBetterWeapon()) NextWeapon();

	// Check for airdrops
	if (curMode != AI_Retreat) {
		int airdroptype = 2;
		if (IsEnemy) airdroptype = 1;
		if (AirDrop::AirDropping == airdroptype && rand()%60 == 0) {
			SetAIMode(AI_Retreat);
			step = 1;
		}
	}

	// Watch
	if (curMode == AI_Watch) {
		Move(Direction*0.3);
		if (timer == 0) {
			if (step % 2 == 1) {
				timer = 50+(rand()%300);
				Direction = -Direction;
			}
			else timer = 15+(rand()%25);
			step++;
		}
		if (timer % 5 == 0) LookForTarget();
		if (step == 10) SetAIMode(AI_Forward);

		// Capture
		if (timer % 10 == 0) {
			Building *building = OverBuilding();
			if (building) {
				if (building->HasOwner && building->IsEnemy != IsEnemy) Capture(building);
			}
		}
	}

	// Charge forward
	else if (curMode == AI_Forward) {
		int dir = 1;
		if (IsEnemy) dir = -1;
		Move(dir);

		// Building
		if (timer % 10 == 0) {
			Building *building = OverBuilding();
			if (building) {
				// Capture
				if (building->HasOwner && building->IsEnemy != IsEnemy) Capture(building);

				// Enter
				if (!Inside) {
					if ((!building->HasOwner || building->IsEnemy == IsEnemy) && !building->NoShoot) {
						if (LeaveIn == -1) {
							building->EnterBuilding(this);
							LeaveIn = 500;
						}
					}
				}
			}
		}

		if (timer == 0) {
			timer = 30;
			LookForTarget();
		}
	}

	// Retreat
	else if (curMode == AI_Retreat) {
		int dir = -1;
		if (IsEnemy) dir = 1;
		Move(dir);

		// Try to take shelter
		if (timer % 5 == 0) {
			Building *building = OverBuilding();
			if (building) {
				if (!building->HasOwner || building->IsEnemy == IsEnemy) {
					building->EnterBuilding(this);
					LeaveIn = -1;
				}
			}
		}

		// Back to patrol
		if (step == 0 && timer == 0) {
			if (Inside) Inside->LeaveBuilding(this);
			SetAIMode(AI_Watch);
			//LookForTarget();
		}

		// Back to patrol (airdrop)
		else if (step == 1) {
			if (timer % 50 == 0) {
				int airdroptype = 2;
				if (IsEnemy) airdroptype = 1;
				if (AirDrop::AirDropping != airdroptype) {
					step = 0;
					timer = 200;
				}
			}
			if (timer == 0) {
				timer = 100;
			}
		}
	}

	// Attack
	else if (curMode == AI_Attack) {
		// Check if target is still alive
		if (!target->Alive || (target->Inside && target->Inside->NoShoot)) {
			SetAIMode(AI_Watch);
			LookForTarget();
		}

		else {
			double delta = target->x - x;
			// Too far
			if (ABS(delta) > FavoriteDistance*4) {
				SetAIMode(AI_Watch);
				LookForTarget();
			}
			else {
				// Look at target
				int dir = 1;
				if (target->x < x) dir = -1;
				Direction = dir;

				// Get closer
				//if (ABS(delta) > FavoriteDistance) Move(dir);
				Move(dir);

				// Shoot step
				if (timer == 0) {
					if (step == 0) {
						step = 1;
						timer = 20;
					}
					else {
						step = 0;
						timer = 40;
					}
				}

				// Shoot it
				if (step == 0) {
					double target_angle = atan2(y-target->y,(target->x-x)*Direction);
					if (target_angle > angle) angle += 0.02;
					if (target_angle < angle) angle -= 0.02;
					angle = MID(-0.5,angle,0.5);
					if (Weapon == 1) angle += 0.5;
					Shoot();
				}

				// Building
				if (timer % 10 == 0) {
					Building *building = OverBuilding();
					if (building) {
						// Capture
						if (building->HasOwner && building->IsEnemy != IsEnemy) Capture(building);

						// Enter
						if (!Inside) {
							if ((!building->HasOwner || building->IsEnemy == IsEnemy) && !building->NoShoot) {
								building->EnterBuilding(this);
								LeaveIn = 500;
							}
						}
					}
				}
			}
		}
	}

	// Decrease timer
	timer--;

	// Update as soldier
	Soldier::Update();
}


///////////////
// Set AI mode
void SoldierAI::SetAIMode(AI_Mode mode) {
	curMode = mode;
	if (curMode == AI_Watch) {
		step = 6;
		timer = 50+(rand()%300);
	}
	if (curMode == AI_Attack) {
		step = 0;
		timer = 40;
	}
	if (curMode == AI_Retreat) {
		step = 0;
		timer = 600;
	}
	if (curMode == AI_Forward) {
		timer = 10;
	}
	if (curMode != AI_Attack) {
		target = NULL;
		angle = 0;
	}
}


//////////////////////
// Looks for a target
void SoldierAI::LookForTarget() {
	// Prepare
	Soldier *best = NULL;
	Soldier *current;
	double closestDist = 1000;
	double curDist;

	// Enemy searches for friendly
	using std::list;
	list<Soldier*>::iterator begin;
	list<Soldier*>::iterator end;
	if (IsEnemy) {
		begin = playFrame->SoldierFriendly.begin();
		end = playFrame->SoldierFriendly.end();
	}
	else {
		begin = playFrame->SoldierEnemy.begin();
		end = playFrame->SoldierEnemy.end();
	}

	// Searches for enemy
	for (std::list<Soldier*>::iterator cur=begin;cur!=end;cur++) {
		current = (*cur);
		if (!current->Alive || (current->Inside && current->Inside->NoShoot)) continue;;
		curDist = ABS(x-current->x);
		if (curDist < FavoriteDistance*4 && curDist < closestDist) {
			closestDist = curDist;
			best = current;
		}
	}

	// If found one, attack it
	if (best) {
		target = best;
		SetAIMode(AI_Attack);
	}
}


///////
// Die
void SoldierAI::Die() {
	// Spawn collectible
	int value = rand()%400;
	int n = -1;
	if (IsEnemy) {
		if (value < 3) n = 3;
		else if (value < 6) n = 4;
		else if (value < 15) n = 2;
		else if (value < 25) n = 1;
		else if (value < 65) n = 0;
	}

	Item *item;
	if (n != -1) {
		item = new Item(x,y,n);
		parent->AddSprite(item);
	}

	// Die as soldier (profound, isn't it? nm. Third day of speedhack, getting tired.)
	Soldier::Die();
}
